/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */



 
/**
 * Etendre un objet avec un extension
 * Hritage javascript pour ajouter des fonctions / mthodes  un objet javascript existant
 * @param  {classe} extension la classe javascript que vous avez crs
 * @param  {class} obj       la classe javascript native
 * 
 */
function extend( extension, obj ){
  for ( var key in extension ){
    obj[key] = extension[key];
  }
}


/**
 * Exemple d'utilisation du modle de conception Decorator.
 * 
 * @author Matthieu
 * 
 */ 



/**
 * La classe MacbookPro par dfaut
 *
 * 
 */
var MacbookPro = function(){
    this.screenSize = 15.6; // taille de l'cran
};


/**
 * Les mthodes de la classe MacbookPro
 * @type {Object}
 */
MacbookPro.prototype = {
    add4GBRam: function(){
      // ajout de 4 Go de RAM
      return "2 Go de RAM par dfaut";
    },
    add8GBRam:function(){
      // ajout de 8 Go de RAM
      return "2 Go de RAM par dfaut";
    },
    addCase: function(){
      // ajout d'un sac de transport
      return "Pas de sac de transport";
    },
    // le descriptif du macbook
    getDetails: function(){
        return "Un MacBook Pro flambant neuf avec un cran de "+this.screenSize+" pouces."; 
    },  
    // le prix du MacbookPro
    getPrice: function(){
      // le prix de base
      return 900.00;         
    }
};



/**
 * La classe Abstraite MacbookDecorator
 * Equivalent  AbstractBoisson de l'exemple ActionScript
 * 
 * @param {classe} macbook instance MacbookPro
 */
var MacbookDecorator = function( macbook ){
    this.macbook = macbook;  

};


/**
 * Les mthodes de la classe Abstraite MacbookDecorator
 * @type {Object}
 */
MacbookDecorator.prototype = {
    add4GBRam: function(){
        return this.macbook.add4GBRam();
    },
    add8GBRam:function(){
        return this.macbook.add8GBRam();
    },
    addCase: function(){
        return this.macbook.addCase();
    },
    // le descriptif du macbook
    getDetails: function(){
        return this.macbook.getDetails(); 
    },  
    // le prix du macbook
    getPrice: function(){
        return this.macbook.getPrice(); 
    }        
};


/**
 * Ajout d'un sac de transport.
 * 
 * @param {classe} macbook instance de la classe MacbookPro
 * 
 */
var CaseDecorator = function( macbook )
{  
   this.macbook = macbook;

};

// ajoute les fonctionnalits CaseDecorator  MacbookDecorator - hrite de la superclass
extend( CaseDecorator, MacbookDecorator ); 

CaseDecorator.prototype.addCase = function(){
    return this.macbook.addCase() + "Sac de transport macbook.";   
};


CaseDecorator.prototype.getDetails = function(){
    return this.macbook.getDetails() + "<br/>Ajout d'un sac de transport.";   
};

CaseDecorator.prototype.getPrice = function(){
    return this.macbook.getPrice() + 45.00;  
};


/**
 * Ajout de 8 Go de RAM.
 * 
 * @param {classe} macbook instance de la classe MacbookPro
 * 
 */
var Boost8GBRam = function( macbook )
{  
   this.macbook = macbook;

};

// ajoute les fonctionnalits Boost8GBRam  MacbookDecorator - hrite de la superclass
extend( Boost8GBRam, MacbookDecorator ); 

Boost8GBRam.prototype.add8GBRam = function(){
    return this.macbook.add8GBRam() + "8 Go de RAM.";   
};

Boost8GBRam.prototype.getDetails = function(){
    return this.macbook.getDetails() + "<br/>Ajout de 8 Go de RAM.";   
};

Boost8GBRam.prototype.getPrice = function(){
    return this.macbook.getPrice() + 145.00;  
};


//-------------------------------------------------
//-------------------------------------------------
//-------------------------------------------------

// Utilisation du pattern Decorator :

console.log("---> exemple javascript Decorator");// avec extension firebug


// Instance de macbook
var oMacbookPro = new MacbookPro();  


// Prix de base : 900.00
console.log( "Dtails du MacBookPro : "+oMacbookPro.getDetails() );
console.log( "Prix du MacBookPro : "+oMacbookPro.getPrice() );
document.write( "Dtails du MacBookPro : "+oMacbookPro.getDetails()+" <br />" );// criture dans le document HTML
document.write( "Prix du MacBookPro : "+oMacbookPro.getPrice()+" euros<br />_______________________<br />");// criture dans le document HTML

// Decorate the macbook - ajout automatique d'un sac de transport
oMacbookPro = new CaseDecorator( oMacbookPro );


// Nouveau prix (avec le sac de transport) 945.00
console.log( "Dtails du MacBookPro : "+oMacbookPro.getDetails() );
console.log( "Prix du MacBookPro : "+oMacbookPro.getPrice() );
document.write( "Dtails du MacBookPro : "+oMacbookPro.getDetails()+" <br />" );// criture dans le document HTML
document.write( "Prix du MacBookPro : "+oMacbookPro.getPrice()+" euros<br />_______________________<br />" );// criture dans le document HTML




// Decorate the macbook - ajout automatique de 8 GO de RAM
oMacbookPro = new Boost8GBRam( oMacbookPro );


// Nouveau prix (avec les 8 Go de RAM) 1090.00
console.log( "Dtails du MacBookPro : "+oMacbookPro.getDetails() );
console.log( "Prix du MacBookPro : "+oMacbookPro.getPrice() );
document.write( "Dtails du MacBookPro : "+oMacbookPro.getDetails()+" <br />" );// criture dans le document HTML
document.write( "Prix du MacBookPro : "+oMacbookPro.getPrice()+" euros<br />_______________________<br />" );// criture dans le document HTML

